using System.Collections.Generic;
using System.Drawing;

using VirtualLabAPI.Core.Common;
using VirtualLabAPI.Core.FieldRepresentations;
using VirtualLabAPI.Core.Numerics;
using VirtualLabAPI.Core.ParameterRuns;
using VirtualLabAPI.Core.ExportImport;

namespace OwnCode {
	/// <summary>
	/// Module for exporting the results of a Parameter Run to hard disc.
	/// </summary>
	/// <remarks>
	/// • Author: LightTrans GmbH and Wyrowski Photonics GmbH
	/// • Version: 1.1, Date: 2020-12-02
	/// • Tested with VirtualLab 2020.2.
	/// • License: CC-BY 3.0
	/// </remarks>
	public class VLModule {
		//enter the indices of the desired detectors: 
		int indexCameraDetector = 611;
		int indexValueDetector = 612;
		string subdetectorValue = "RMS [λ] of Wavefront Error";

		//enter the desired directory and name of file for export:
		string directory = @"C:\Temp\Results\";
		string filename = "file_";

		public void Run() {
			int dummy;
			//get parameter run
			ParameterRun prToExport = Globals.ActiveDocumentHistory.BrowseLastDocuments(DocumentFilter.ParameterRun,
																						out dummy,
																						"Select PR for Export",
																						true) as ParameterRun;

			//error handling
			if (prToExport == null) {
				return;
			}

			string detectorName = "";
			string detectorName2 = "";

			#region search row index for specificed camera detector
			//search for index of result
			for (int runSearchIndex = 0; runSearchIndex < prToExport.ResultMatrix.NameOfResults.Count; runSearchIndex++) {
				//get current string of result
				string currResultCaption = prToExport.ResultMatrix.NameOfResults[runSearchIndex];

				//check whether index of camera detector is within caption => then stop and store index
				if (currResultCaption.Contains("#" + indexCameraDetector.ToString())) {
					//store index for export
					detectorName = currResultCaption;
					//stop loop
					//break;
				}
				if (currResultCaption.Contains("#" + indexValueDetector.ToString())) {
					//store index for export
					detectorName2 = currResultCaption;
					//stop loop
					//break;
				}
			}
			#endregion

			if (detectorName.Equals("")) {
				Globals.DataDisplay.LogError("No detector with corresponding index found!");
				return;
			}

			if (detectorName2.Equals("")) {
				Globals.DataDisplay.LogError("No detector with corresponding index found!");
				return;
			}
			//#endregion

			List<IElementaryParameterRunResult> listResults;
			List<IElementaryParameterRunResult> listResults2;
			List<PhysicalValue> iterationHeaders;
			List<PhysicalValue> iterationHeaders2;
			prToExport.ResultMatrix.GetResultsOfSpecificDetector(detectorName, "", 0, prToExport.ResultMatrix.NumberOfIterations - 1, out iterationHeaders, out listResults);
			prToExport.ResultMatrix.GetResultsOfSpecificDetector(detectorName2, subdetectorValue, 0, prToExport.ResultMatrix.NumberOfIterations - 1, out iterationHeaders2, out listResults2);


			//loop over all results for both Detectors
			for (int runResults = 0; runResults < listResults.Count; runResults++) {
				//extract current result
				IElementaryParameterRunResult currResult = listResults[runResults];
				IElementaryParameterRunResult currResult2 = listResults2[runResults];

				//if result is not available => skip
				if (currResult == null) {
					continue;
				}

				//convert as chromatic fields set
				ChromaticFieldsSet resultAsCFS = currResult.ActualParameterRunResult as ChromaticFieldsSet;

				DataArray2D daSumOfAllSubsets = resultAsCFS.GetSumOfAllSubsets() as DataArray2D;

				//generate complex amplitude from given complex field
				ComplexAmplitude caField = new ComplexAmplitude(daSumOfAllSubsets.Data[0]);
				//set correct sampling parameters
				caField.SamplingDistance = new VectorD(daSumOfAllSubsets.SamplingDistance_X, daSumOfAllSubsets.SamplingDistance_Y);

				double min, max;
				//get min/max
				ComplexFieldEvaluation.GetMinMaxValues(daSumOfAllSubsets.Data[0],
													   FieldQuantity.RealPart,
													   out min,
													   out max);

				string resultValue = (currResult2.ActualParameterRunResult as PhysicalValue).ToString();

				//generate colormapping object
				ColorMapping cMapping = new ColorMapping();
				cMapping.DefineFieldQuantity(FieldQuantity.RealPart, min, max, startColor: Color.Black, endColor: Color.White, colorFromWavelength: false);

				//generate filename
				string filenameBase = directory + filename;
				int numberOfFile = runResults + 1;
				string currFileNameBMP = filenameBase + numberOfFile.ToString("0000") + ".bmp";

				//perform export to BMP
				ImportExport.ExportBitmap(caField,
										  currFileNameBMP,
										  cMapping);
				//generate filename
				string currFileNameTXT = filenameBase + numberOfFile.ToString("0000") + ".txt";
				//generate header
				string header = "# Origin: Simulation for " + iterationHeaders[runResults].ToString("F") + "\r\n";

				header += "# " + subdetectorValue + ": " + resultValue + "\r\n";

				TextExportImportSettings exportSettings = new TextExportImportSettings();
				exportSettings.ColumnSeparator = '\t';
				exportSettings.DecimalSeparator = '.';
				exportSettings.HeaderIndicator = '#';
				header += daSumOfAllSubsets.HeaderInformation(0, exportSettings, FieldQuantity.RealPart);

				//perform txt export
				ImportExportNumerics.ExportComplexField(daSumOfAllSubsets.Data[0], header, FieldQuantity.RealPart, 1, '.', '\t', currFileNameTXT);
			}
		}
	}
}
